unit QButtonEx2;

interface

uses
  Windows, Messages, SysUtils, Classes, QControls, QStdCtrls, Qt;

type
  TQtEvent = procedure(Sender: TComponent; Widget: QObjectH; Event: QEventH) of object;

  TButtonEx2 = class(TButton)
  private
    FOnClickSignal: TNotifyEvent;
    FOnQtEvent: TQtEvent;
  protected
    procedure ClickedSlot; cdecl;
    procedure CreateWidget; override;
    procedure DestroyWidget; override;
    function EventFilter(Sender: QObjectH; Event: QEventH): Boolean; override;
    procedure HookEvents; override;
  published
    property OnClickSignal: TNotifyEvent read FOnClickSignal write FOnClickSignal;
    property OnQtEvent: TQtEvent read FOnQtEvent write FOnQtEvent;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TButtonEx2]);
end;

{ TButtonEx2 }

procedure TButtonEx2.ClickedSlot;
begin
  if Assigned(OnClickSignal) then
    OnClickSignal(Self)
end;

procedure TButtonEx2.CreateWidget;
begin
  //The inherited CreateWidget creates a pushbutton widget, and a
  //button hook object, so we do not need to do anything
  inherited;
  //If this were a component based on a new widget, we would:
  // 1) create the widget and assign its handle
  //    to the component's Handle property, then
  // 2) create a hook object and assign it to the Hooks property.

  //Hooks := QPushButton_hook_create(Handle);

  //As it is, we already get a suitable widget and hook object
  //by calling the inherited version of CreateWidget
end;

procedure TButtonEx2.DestroyWidget;
begin
  inherited;
  //Here we would destroy the widget and hook object,
  //if we were responsible for creating them
end;

function TButtonEx2.EventFilter(Sender: QObjectH; Event: QEventH): Boolean;
begin
  if Assigned(OnQtEvent) then
    OnQtEvent(Self, Sender, Event);
  //Return True if you want to hide the event
  Result := inherited EventFilter(Sender, Event)
end;

procedure TButtonEx2.HookEvents;
var
  Method: TMethod;
begin
  inherited;
  QButton_clicked_Event(Method) := ClickedSlot;
  QButton_hook_hook_clicked(QButton_hookH(Hooks), Method);
end;

end.
